/* -*- C++ -*-
 *
 * testNewCode.cpp
 *
 * Copyright (c) 2002 CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 * $Id: sample.cpp,v 1.5 2003/10/03 14:30:38 mika Exp $
 *
 */


#include "EdgReplicaMetadataCatalog/ReplicaMetadataCatalogFactory.h"
#include "EdgReplicaMetadataCatalog/ReplicationExceptions.h"
#include <vector>
#include <iostream>

#ifdef WITH_WP2_SECURITY
#include <WP2Security.h>
// Defs for getopt
char optchar, *optarg;
int optind, optopt;
int opterr = 1;

void usage(char *);

#endif

int main( int argc, char * const argv[]) 
{
  using namespace EdgReplicaMetadataCatalog;

  ReplicaMetadataCatalog *rmc = NULL;

#ifdef WITH_WP2_SECURITY
  // Let's see what args were given
  char *credentials = NULL;
  char *passphrase = NULL;
  char *cacertdir = NULL;
  char *rseedfile = NULL;
  int use_security = 1;
  char *role = NULL;
  char *policy = NULL;

  char *accessurl = "https://localhost:8443/wpsix/edg-replica-metadata-catalog/services/edg-replica-metadata-catalog";

  if (argc > 0) {
    while ((optchar = getopt(argc, argv, "c:p:d:r:u:R:P:nhv")) > 0) {
      switch (optchar) {
      case 'c':
	// File containing user's public credentials (cert+key)
	credentials = optarg;
	use_security = 1;
	break;
      case 'p':
	// User's private passphrase
	passphrase = optarg;
	use_security = 1;
	break;
      case 'd':
	// Path to the dir of trusted CA certs
	cacertdir = optarg;
	use_security = 1;
	break;
      case 'r':
	// Random seed file
	rseedfile = optarg;
	use_security = 1;
	break;
      case 'u':
	// Access url defining the service
	accessurl = optarg;
	if (!strncmp("https:",accessurl,6))
	  use_security = 1;
	break;
      case 'n':
	// Don't use security
	use_security = 0;
	break;
      case 'R':
	// Define role
	use_security = 1;
	role = optarg;
	break;
      case 'P':
	// Define policy
	use_security = 1;
	policy = optarg;
	break;
      default:
      case 'v':
      case 'h':
	usage(argv[0]);
	exit(1);
      }
    }
  }
  
  const std::string theBoltedUrl(accessurl);

  if (use_security)
    rmc = ReplicaMetadataCatalogFactory::getReplicaMetadataCatalog(theBoltedUrl,
								   passphrase,
								   credentials,
								   cacertdir,
								   rseedfile);
  else
    rmc = ReplicaMetadataCatalogFactory::getReplicaMetadataCatalog(theBoltedUrl);

#else

    rmc = ReplicaMetadataCatalogFactory::getReplicaMetadataCatalog(
	   "http://localhost:8080/wpsix/edg-replica-metadata-catalog/services/edg-replica-metadata-catalog");

#endif
  
  //const std::string guid1("guid:1");
  //
  // add a mapping, and then retrieve the guid from the pfn
  //try {
  //  lrc->addMapping(guid1, "pfn1://foo/bar");
  //} catch(PfnExistsException e) {
  //  // ignore it, mapping exists alread
  //}
  //
  //const std::string pfn("pfn1://foo/bar");
  //const std::string guid = lrc->guidForPfn(pfn);
  //std::cout << "Got GUID: " << guid << std::endl;

  /*
  try {
    lrc->addAlias("guid:1", "lfn://alias1");
  } catch (AliasExistsException e) {
    // ignore it, alias already exists
    std::cout << "Caught AliasExistsException" << std::endl;
  }catch (NoSuchGuidException e) {
    std::cout << "Tried to add an alias to a guid that didn't exist" << 
      guid1 << std::endl;
    delete lrc;
    return 1;
  }
  */

  delete rmc;

  return 0;
} 


void usage(char *pname) {
#ifdef WITH_WP2_SECURITY
  std::cerr << "Usage: " << pname << " [ options ]" << std::endl;
  std::cerr << "This product includes software developed by the OpenSSL ";
  std::cerr << "Project\nfor use in the OpenSSL Toolkit. (http://www.openssl.org/)";
  std::cerr << std::endl << std::endl << "Options:" << std::endl;
  std::cerr << "-c your_public_cert+key_file" << std::endl;
  std::cerr << "-d dir_of_trusted_CA_certs" << std::endl;
  std::cerr << "-p your_private_keys_passphrase" << std::endl;
  std::cerr << "-r path_to_random_seed_file" << std::endl;
  std::cerr << "-R role" << std::endl;
  std::cerr << "-P policy" << std::endl;
  std::cerr << "-u service_url" << std::endl;
  std::cerr << "-n (don't use security features)" << std::endl;
  std::cerr << "-h (prints help and version number)" << std::endl;
  std::cerr << "-v (prints help and version number)" << std::endl;
  std::cerr << std::endl << "Defaults:" << std::endl;
  std::cerr << "c = /tmp/x509up_uUID (i.e. your user proxy cert)" << std::endl;
  std::cerr << "d = /etc/grid-security/certificates" << std::endl;
  std::cerr << "r = ~/.rnd" << std::endl;
  std::cerr << "u = (a hard coded url in the sample application)" << std::endl;
  std::cerr << "R = myDefaultRole" << std::endl;
  std::cerr << "P = myDefaultPolicy" << std::endl << std::endl;
  std::cerr << "It's better to run grid-proxy-init than using long term " << std::endl;
  std::cerr << "credentials with -c and -p." << std::endl;
#else
  std::cerr << "Usage: " << pname << std::endl;
#endif
}
